# -*- coding: utf-8 -*-
# pylint: disable=expression-not-assigned,line-too-long
"""Working hours (Danish arbejdstimer) or not? API model."""
from __future__ import annotations

from datetime import date
from enum import Enum
from typing import Annotated, List, Optional, no_type_check

from pydantic import BaseModel, Field, validator


class Application(Enum):
    arbejdstimer = 'arbejdstimer'


class Operator(Enum):
    and_ = 'and'
    or_ = 'or'
    xor = 'xor'


class Hour(BaseModel):
    __root__: Annotated[int, Field(ge=0.0, le=23.0)]


class WorkingHours(BaseModel):
    __root__: Annotated[
        List[Hour],
        Field(
            description='Inclusive range of 24 hour start and end integer values.',
            max_items=2,
            min_items=2,
            title='Working Hours',
        ),
    ]


class Dates(BaseModel):
    __root__: Annotated[
        List[date],
        Field(
            description='Two dates are an inclusive range and 1, 3, or more dates represent a set of dates.',
            min_items=1,
            title='Dates - Range or Set',
        ),
    ]

    @no_type_check
    @validator('__root__')
    def is_unique(cls, v):
        if v and 1 < len(v) != len(set(v)):
            raise ValueError('dates must be unique')
        return v


class Holiday(BaseModel):
    label: Annotated[Optional[str], Field(examples=['public holiday'])] = ''
    at: Dates


class Holidays(BaseModel):
    __root__: Annotated[
        List[Holiday],
        Field(
            description='Optionally labeled dates of non-working days.',
            min_items=0,
            title='Holidays',
        ),
    ]


class Arbejdstimer(BaseModel):
    api: Annotated[
        Optional[int],
        Field(
            description='API version of the application this configuration targets.',
            title='API Version',
        ),
    ] = 1
    application: Annotated[
        Optional[Application],
        Field(
            description='Name of the application this configuration targets.',
            title='Application Name',
        ),
    ] = Application.arbejdstimer
    operator: Annotated[
        Operator,
        Field(
            description='Logic combining the given specific values with the application defaults.',
            title='Logic for Combination with Default Values',
        ),
    ]
    holidays: Optional[Holidays] = None
    working_hours: Optional[WorkingHours] = None



# -*- coding: utf-8 -*-
# pylint: disable=line-too-long,missing-docstring,reimported,unused-import,unused-variable
import datetime as dti
import json
import pathlib

import arbejdstimer.arbejdstimer as at

CFG_FS_EMPTY = pathlib.Path('tests', 'fixtures', 'basic', 'minimal-config.json')
CFG_FS_META_ONLY = pathlib.Path('tests', 'fixtures', 'basic', 'meta-only-config.json')
CFG_FS_HOLIDAYS = pathlib.Path('tests', 'fixtures', 'basic', 'holidays-config.json')
CFG_FS_TRIPLET_HOLIDAYS = pathlib.Path('tests', 'fixtures', 'basic', 'triplet-holidays-config.json')
CFG_FS_NOT_THERE = pathlib.Path('does', 'not', 'exist', 'hypothetical.json')
CFG_FS_NO_JSON_EXTENSION = pathlib.Path('tests', 'fixtures', 'basic', 'this-has-no-json-extens.ion')
CFG_FS_INVALID_MINIMAL = pathlib.Path('tests', 'fixtures', 'basic', 'invalid-minimal-config.json')
CFG_PY_EMPTY = {
    'operator': 'or',
}

TODAY = dti.date.today()
CFG_PY_TODAY_HOLIDAY = {
    'api': 1,
    'application': 'arbejdstimer',
    'operator': 'or',
    'holidays': [{'at': [TODAY.strftime(at.DATE_FMT)]}],
    'working_hours': [8, 17],
}

ENCODING = 'utf-8'
with open(CFG_FS_HOLIDAYS, 'rt', encoding=ENCODING) as handle:
    CFG_PY_HOLIDAYS = json.load(handle)

with open(CFG_FS_TRIPLET_HOLIDAYS, 'rt', encoding=ENCODING) as handle:
    CFG_PY_TRIPLET_HOLIDAYS = json.load(handle)


def always_monday(date: dti.date) -> int:  # type: ignore
    """Return current weekday mock for Monday."""
    return 1


def always_sunday(date: dti.date) -> int:  # type: ignore
    """Return current weekday mock for Sunday."""
    return 7


def never_weekend(day_number: int) -> bool:  # type: ignore
    """Return if day number is weekend mock for never."""
    return True


def always_weekend(day_number: int) -> bool:  # type: ignore
    """Return if day number is weekend mock for always."""
    return False


def the_zero_hour() -> int:
    """Return the hour of day as integer within [0, 23] mock 0."""
    return 0


def the_noon_hour() -> int:
    """Return the hour of day as integer within [0, 23] mock 12."""
    return 12



# -*- coding: utf-8 -*-
# pylint: disable=expression-not-assigned,line-too-long
"""Working hours (Danish arbejdstimer) or not? API."""
import datetime as dti
import json
import os
import pathlib
import sys
from typing import List, Optional, Tuple, Union, no_type_check

from pydantic.error_wrappers import ValidationError

import arbejdstimer.api as api

DEBUG_VAR = 'ARBEJDSTIMER_DEBUG'
DEBUG = os.getenv(DEBUG_VAR)

ENCODING = 'utf-8'
ENCODING_ERRORS_POLICY = 'ignore'

DEFAULT_CONFIG_NAME = '.arbejdstimer.json'
CFG_TYPE = dict[str, Union[dict[str, str], list[dict[str, Union[str, list[str]]]]]]
WORKING_HOURS_TYPE = Union[tuple[int, int], tuple[None, None]]
DATE_FMT = '%Y-%m-%d'
DEFAULT_WORK_HOURS_MARKER = (None, None)


def weekday(date: dti.date) -> int:
    """Return current weekday."""
    return date.isoweekday()


def no_weekend(day_number: int) -> bool:
    """Return if day number is weekend."""
    return day_number < 6


def the_hour() -> int:
    """Return the hour of day as integer within [0, 23]."""
    return dti.datetime.now().hour


@no_type_check
def apply(off_days: list[dti.date], working_hours: WORKING_HOURS_TYPE, cmd: str) -> Tuple[int, str]:
    """Apply the effective rules to the current date and time."""
    working_hours = working_hours if working_hours != (None, None) else (7, 16)
    today = dti.date.today()
    if today not in off_days:
        if cmd == 'explain':
            print(f'- Today ({today}) is not a holiday')
    else:
        return 1, '- Today is a holiday.'

    work_day = no_weekend(weekday(today))
    if work_day:
        if cmd == 'explain':
            print(f'- Today ({today}) is not a weekend')
    else:
        return 1, '- Today is weekend.'

    hour = the_hour()
    if working_hours[0] <= hour <= working_hours[1]:
        if cmd == 'explain':
            print(f'- At this hour ({hour}) is work time')
    else:
        return 1, f'- No worktime at hour({hour}).'

    return 0, ''


@no_type_check
def load(cfg: CFG_TYPE) -> Tuple[int, str, list[dti.date], WORKING_HOURS_TYPE]:
    """Load the configuration and return error, message and holidays as well as working hours list.

    The holidays as well as non-default working hours will be ordered.
    """
    if not cfg:
        return 0, 'empty configuration, using default', [], DEFAULT_WORK_HOURS_MARKER

    try:
        model = api.Arbejdstimer(**cfg)
    except ValidationError as err:
        return 2, str(err), [], (None, None)

    holidays_date_list = []
    if model.holidays:
        holidays = model.dict()['holidays']
        for nth, holiday in enumerate(holidays, start=1):
            dates = holiday['at']
            if len(dates) == 1:
                holidays_date_list.append(dates[0])
            elif len(dates) == 2:
                data = sorted(dates)
                start, end = [data[n] for n in (0, 1)]
                current = start
                holidays_date_list.append(current)
                while current < end:
                    current += dti.timedelta(days=1)
                    holidays_date_list.append(current)
            else:
                for a_date in dates:
                    holidays_date_list.append(a_date)

    working_hours = DEFAULT_WORK_HOURS_MARKER
    if model.working_hours:
        working_hours = tuple(sorted(model.working_hours.dict().get('__root__', [None, None])))
    return 0, '', sorted(holidays_date_list), working_hours


def verify_request(argv: Optional[List[str]]) -> Tuple[int, str, List[str]]:
    """Fail with grace."""
    if not argv or len(argv) != 2:
        return 2, 'received wrong number of arguments', ['']

    command, config = argv

    if command not in ('explain', 'now'):
        return 2, 'received unknown command', ['']

    if not config:
        return 2, 'configuration missing', ['']

    config_path = pathlib.Path(str(config))
    if not config_path.is_file():
        return 1, f'config ({config_path}) is no file', ['']
    if not ''.join(config_path.suffixes).lower().endswith('.json'):
        return 1, 'config has not .json extension', ['']

    return 0, '', argv


def main(argv: Union[List[str], None] = None) -> int:
    """Drive the lookup."""
    error, message, strings = verify_request(argv)
    if error:
        print(message, file=sys.stderr)
        return error

    command, config = strings

    with open(config, 'rt', encoding=ENCODING) as handle:
        configuration = json.load(handle)

    error, message, holidays, working_hours = load(configuration)
    if error:
        if command == 'explain':
            print(message, file=sys.stderr)
        return error
    if command == 'explain':
        print(f'read valid configuration from ({config})')

    if command == 'explain':
        print(f'consider {len(holidays)} holidays:')
    error, message = apply(holidays, working_hours, command)
    if error:
        if command == 'explain':
            print(message, file=sys.stdout)
        return error

    return 0



# -*- coding: utf-8 -*-
# pylint: disable=line-too-long,missing-docstring,reimported,unused-import,unused-variable
import datetime as dti

import pytest
from pydantic.error_wrappers import ValidationError

import arbejdstimer.api as api
import arbejdstimer.arbejdstimer as at
import tests.conftest as fix


def _subs(count: int, what: str) -> str:
    """DRY."""
    return f'{count} validation error{"" if count == 1 else "s"} for {what}'


def test_api_hour_noon():
    twelve = api.Hour(__root__='12')  # type: ignore
    assert twelve.__root__ == 12


def test_api_hour_42():
    with pytest.raises(ValidationError, match=_subs(1, 'Hour')) as err:
        _ = api.Hour(__root__='42')  # type: ignore
    assert '\n__root__\n  ensure this value is less than or equal to 23' in str(err.value)


def test_api_hour_negative():
    with pytest.raises(ValidationError, match=_subs(1, 'Hour')) as err:
        _ = api.Hour(__root__='-1')  # type: ignore
    assert '\n__root__\n  ensure this value is greater than or equal to 0' in str(err.value)


def test_api_hour_no_digit():
    with pytest.raises(ValidationError, match=_subs(1, 'Hour')) as err:
        _ = api.Hour(__root__='_')  # type: ignore
    assert '\n__root__\n  value is not a valid integer' in str(err.value)


def test_api_date_range_wun():
    wun = api.Dates(__root__=['2021-12-31'])  # type: ignore
    assert wun.__root__ == [dti.date(2021, 12, 31)]


def test_api_date_range_two():
    two = api.Dates(__root__=['2021-12-31', '2022-01-01'])  # type: ignore
    assert two.__root__ == [dti.date(2021, 12, 31), dti.date(2022, 1, 1)]


def test_api_date_range_three():
    three = api.Dates(__root__=['2021-12-31', '2022-01-01', '2022-01-02'])  # type: ignore
    assert three.__root__ == [dti.date(2021, 12, 31), dti.date(2022, 1, 1), dti.date(2022, 1, 2)]


def test_api_date_range_duplicate():
    with pytest.raises(ValidationError, match=_subs(1, 'Dates')) as err:
        _ = api.Dates(__root__=['2021-12-31', '2021-12-31'])  # type: ignore
    assert '\n__root__\n  dates must be unique' in str(err.value)


def test_api_working_hours_nine_to_five():
    nine = api.Hour(__root__='9')  # type: ignore
    five = api.Hour(__root__='17')  # type: ignore
    assert api.WorkingHours(__root__=[nine, five]).__root__ == [nine, five]


def test_api_working_hours_empty_list():
    with pytest.raises(ValidationError, match=_subs(1, 'WorkingHours')) as err:
        _ = api.WorkingHours(__root__=[])  # type: ignore
    assert '\n__root__\n  ensure this value has at least 2 items' in str(err.value)


def test_api_working_hours_nine_single_value():
    nine = api.Hour(__root__='9')  # type: ignore
    with pytest.raises(ValidationError, match=_subs(1, 'WorkingHours')) as err:
        _ = api.WorkingHours(__root__=[nine])  # type: ignore
    assert '\n__root__\n  ensure this value has at least 2 items' in str(err.value)


def test_api_working_hours_nine_nine_five_triplet():
    nine = api.Hour(__root__='9')  # type: ignore
    five = api.Hour(__root__='17')  # type: ignore
    with pytest.raises(ValidationError, match=_subs(1, 'WorkingHours')) as err:
        _ = api.WorkingHours(__root__=[nine, nine, five])  # type: ignore
    assert '\n__root__\n  ensure this value has at most 2 items' in str(err.value)


def test_api_holiday_wun():
    wun = api.Dates(__root__=['2021-12-31'])  # type: ignore
    holiday = api.Holiday(at=wun)
    assert holiday.at.__root__ == [dti.date(2021, 12, 31)]
    assert holiday.json() == '{"label": "", "at": ["2021-12-31"]}'
    data = {'label': '', 'at': ['2021-12-31']}
    # noinspection Pydantic
    another_holiday = api.Holiday(**data)  # type: ignore
    assert holiday == another_holiday


def test_api_today():
    # noinspection Pydantic
    cfg = api.Arbejdstimer(**fix.CFG_PY_TODAY_HOLIDAY)
    today_rep = fix.TODAY.strftime(at.DATE_FMT)
    expected = (
        '{"api": 1, "application": "arbejdstimer", "operator": "or",'
        f' "holidays": [{{"label": "", "at": ["{today_rep}"]}}], "working_hours": [8, 17]}}'
    )
    assert cfg.json() == expected



# -*- coding: utf-8 -*-
# pylint: disable=line-too-long,missing-docstring,reimported,unused-import,unused-variable
import arbejdstimer.arbejdstimer as at
import tests.conftest as fix


def _subs(count: int, what: str) -> str:
    """DRY."""
    return f'{count} validation error{"" if count == 1 else "s"} for {what}'


def test_at_main_now():
    assert at.main(['now', str(fix.CFG_FS_HOLIDAYS)]) in (0, 1)


def test_at_main_now_meta_only():
    assert at.main(['now', str(fix.CFG_FS_META_ONLY)]) in (0, 1)


def test_at_main_now_load_error_processing(capsys):
    assert at.main(['now', str(fix.CFG_FS_INVALID_MINIMAL)]) == 2
    out, err = capsys.readouterr()
    assert not out
    assert not err


def test_at_main_explain():
    assert at.main(['explain', str(fix.CFG_FS_HOLIDAYS)]) in (0, 1)


def test_at_main_explain_load_error_processing(capsys):
    assert at.main(['explain', str(fix.CFG_FS_INVALID_MINIMAL)]) == 2
    out, err = capsys.readouterr()
    assert not out
    message_part = (
        '1 validation error for arbejdstimer\noperator\n  value is not a valid enumeration member;'
        " permitted: 'and', 'or', 'xor'"
    )
    assert message_part in err.lower()


def test_at_verify_request_too_few():
    assert at.verify_request([1]) == (2, 'received wrong number of arguments', [''])  # type: ignore


def test_at_verify_request_unknown_command():
    assert at.verify_request(['unknown', 'does not matter']) == (2, 'received unknown command', [''])


def test_at_verify_request_falsy_input():
    assert at.verify_request(['now', '']) == (2, 'configuration missing', [''])


def test_at_main_holidays(capsys):
    assert at.main(['explain', str(fix.CFG_FS_HOLIDAYS)]) in (0, 1)
    out, err = capsys.readouterr()
    assert 'consider 11 holidays' in out.lower()
    assert not err


def test_at_load_empty():
    assert at.load({}) == (0, 'empty configuration, using default', [], at.DEFAULT_WORK_HOURS_MARKER)


def test_at_load_falsy():
    assert at.load(None) == (0, 'empty configuration, using default', [], at.DEFAULT_WORK_HOURS_MARKER)  # type: ignore


def test_at_load_no_meta():
    message = '1 validation error for Arbejdstimer\noperator\n  field required (type=value_error.missing)'
    assert at.load({'a': 'b'}) == (2, message, [], at.DEFAULT_WORK_HOURS_MARKER)  # type: ignore


def test_at_load_alien_meta():
    code, message, holidays, hours = at.load({'operator': 'or', 'application': None})  # type: ignore
    assert code == 0
    assert message == ''
    assert holidays == []
    assert hours == at.DEFAULT_WORK_HOURS_MARKER


def test_at_load_alien_application():
    code, message, holidays, hours = at.load({'operator': 'or', 'application': 'b'})  # type: ignore
    assert code == 2
    expected_part = (
        '1 validation error for Arbejdstimer\napplication\n  value is not a valid enumeration member;'
        " permitted: 'arbejdstimer'"
    )
    assert expected_part in message
    assert holidays == []
    assert hours == at.DEFAULT_WORK_HOURS_MARKER


def test_at_load_alien_api_version():
    cfg = {'operator': 'or', 'application': 'b', 'api': None}
    code, message, holidays, hours = at.load(cfg)  # type: ignore
    assert code == 2
    expected_part = (
        '1 validation error for Arbejdstimer\napplication\n  value is not a valid enumeration member;'
        " permitted: 'arbejdstimer'"
    )
    assert expected_part in message
    assert holidays == []
    assert hours == at.DEFAULT_WORK_HOURS_MARKER

    cfg = {'operator': 'dirac', 'application': 'arbejdstimer', 'api': 1}
    code, message, holidays, hours = at.load(cfg)  # type: ignore
    assert code == 2
    expected_part = (
        '1 validation error for Arbejdstimer\noperator\n  value is not a valid enumeration member;'
        " permitted: 'and', 'or', 'xor'"
    )
    assert expected_part in message
    assert holidays == []
    assert hours == at.DEFAULT_WORK_HOURS_MARKER

    cfg = {'operator': 42, 'application': 'arbejdstimer', 'api': 1}
    code, message, holidays, hours = at.load(cfg)  # type: ignore
    assert code == 2
    expected_part = (
        '1 validation error for Arbejdstimer\noperator\n  value is not a valid enumeration member;'
        " permitted: 'and', 'or', 'xor'"
    )
    assert expected_part in message
    assert holidays == []
    assert hours == at.DEFAULT_WORK_HOURS_MARKER


def test_at_load_no_holidays_alien_or_empty():
    cfg = {'operator': 'or', 'application': 'arbejdstimer', 'api': 1}
    code, message, holidays, hours = at.load(cfg)  # type: ignore
    assert code == 0
    assert message == ''
    assert holidays == []
    assert hours == at.DEFAULT_WORK_HOURS_MARKER

    cfg = {**cfg, 'holidays': None}
    code, message, holidays, hours = at.load(cfg)  # type: ignore
    assert code == 0
    assert message == ''
    assert holidays == []
    assert hours == at.DEFAULT_WORK_HOURS_MARKER

    cfg = {**cfg, 'holidays': []}
    code, message, holidays, hours = at.load(cfg)  # type: ignore
    assert code == 0
    assert message == ''
    assert holidays == []
    assert hours == at.DEFAULT_WORK_HOURS_MARKER


def test_at_load_holidays_alien():
    cfg = {'operator': 'or', 'application': 'arbejdstimer', 'api': 1}
    cfg = {**cfg, 'holidays': {'holi': 'days'}}
    code, message, holidays, hours = at.load(cfg)  # type: ignore
    assert code == 2
    expected_part = '1 validation error for Arbejdstimer\nholidays -> __root__\n  value is not a valid list'
    assert expected_part in message
    assert holidays == []
    assert hours == at.DEFAULT_WORK_HOURS_MARKER

    cfg = {**cfg, 'holidays': 42}
    code, message, holidays, hours = at.load(cfg)  # type: ignore
    assert code == 2
    assert expected_part in message
    assert holidays == []
    assert hours == at.DEFAULT_WORK_HOURS_MARKER


def test_at_load_holidays_missing_date_range():
    cfg = {'operator': 'or', 'application': 'arbejdstimer', 'api': 1}
    cfg = {**cfg, 'holidays': [{'at': ['ignore']}, {}]}
    code, message, holidays, hours = at.load(cfg)  # type: ignore
    assert code == 2
    expected_parts = (
        '2 validation errors for Arbejdstimer',
        'holidays -> __root__ -> 0 -> at -> __root__ -> 0',
        'invalid date format',
        'holidays -> __root__ -> 1 -> at',
        'field required',
    )
    for part in expected_parts:
        assert part in message
    assert holidays == []
    assert hours == at.DEFAULT_WORK_HOURS_MARKER


def test_at_load_and_apply_today_holiday(capsys):
    error, message, holidays, _ = at.load(fix.CFG_PY_TODAY_HOLIDAY)
    assert not error
    assert not message
    assert at.apply(holidays, (None, None), 'explain') == (1, '- Today is a holiday.')
    out, err = capsys.readouterr()
    assert not out
    assert not err


def test_at_load_triplet_holidays():
    error, message, holidays, hours = at.load(fix.CFG_PY_TRIPLET_HOLIDAYS)
    assert not error
    assert not message
    assert len(holidays) == 3
    assert hours == tuple(fix.CFG_PY_TRIPLET_HOLIDAYS['working_hours'])


def test_at_apply_now_monday_noon():
    at.weekday = fix.always_monday
    at.the_hour = fix.the_noon_hour
    assert at.apply([], at.DEFAULT_WORK_HOURS_MARKER, 'now') == (0, '')


def test_at_apply_now_sunday_noon():
    at.weekday = fix.always_sunday
    at.the_hour = fix.the_noon_hour
    error, message = at.apply([], at.DEFAULT_WORK_HOURS_MARKER, 'now')
    assert error == 1
    assert message == '- Today is weekend.'


def test_at_apply_now_monday_midnight():
    at.weekday = fix.always_monday
    at.the_hour = fix.the_zero_hour
    assert at.apply([], at.DEFAULT_WORK_HOURS_MARKER, 'now') == (1, f'- No worktime at hour({at.the_hour()}).')


def test_at_apply_now_sunday_midnight():
    at.weekday = fix.always_sunday
    at.the_hour = fix.the_zero_hour
    error, message = at.apply([], at.DEFAULT_WORK_HOURS_MARKER, 'now')
    assert error == 1
    assert message == '- Today is weekend.'


def test_at_apply_explain_monday_noon(capsys):
    at.weekday = fix.always_monday
    at.the_hour = fix.the_noon_hour
    assert at.apply([], at.DEFAULT_WORK_HOURS_MARKER, 'explain') == (0, '')
    out, err = capsys.readouterr()
    assert f'- Today ({fix.TODAY}) is not a holiday' in out
    assert f'- At this hour ({at.the_hour()}) is work time' in out
    assert not err



# -*- coding: utf-8 -*-
# pylint: disable=line-too-long,missing-docstring,reimported,unused-import,unused-variable
import click
import pytest

import arbejdstimer.cli as cli
import tests.conftest as fix


def test_main_legacy_ok(capsys):
    assert cli.main(['explain', str(fix.CFG_FS_HOLIDAYS)]) in (0, 1)
    out, err = capsys.readouterr()
    assert 'read valid configuration from (' in out.lower()
    assert not err


def test_version_ok(capsys):
    with pytest.raises(click.exceptions.Exit) as exec_info:
        assert cli.app_version() == 0
    assert exec_info.value.exit_code == 0
    out, err = capsys.readouterr()
    assert 'version' in out.lower()
    assert not err


def test_now_ok(capsys):
    with pytest.raises(SystemExit) as exec_info:
        cli.now(conf=fix.CFG_FS_EMPTY)  # type: ignore
    assert exec_info.value.code in (0, 1)
    out, err = capsys.readouterr()
    assert not out
    assert not err


def test_explain_ok(capsys):
    with pytest.raises(SystemExit) as exec_info:
        cli.explain(conf=fix.CFG_FS_EMPTY)  # type: ignore
    assert exec_info.value.code in (0, 1)
    out, err = capsys.readouterr()
    assert 'read valid configuration from (' in out.lower()
    assert 'consider 0 holidays' in out.lower()
    assert not err


def test_template_ok(capsys):
    with pytest.raises(SystemExit) as exec_info:
        cli.app_template()
    assert exec_info.value.code == 0
    out, err = capsys.readouterr()
    assert 'company holidays 2021/2022' in out.lower()
    assert not err


def test_at_main_explain_meta_only(capsys):
    with pytest.raises(SystemExit) as exec_info:
        cli.explain(conf=fix.CFG_FS_META_ONLY)
    assert exec_info.value.code in (0, 1)
    out, err = capsys.readouterr()
    message_part = 'read valid configuration from (tests/fixtures/basic/meta-only-config.json)\nconsider 0 holidays:'
    assert message_part in out.lower()
    assert not err


def test_non_existing_configuration_file(capsys):
    with pytest.raises(SystemExit) as exec_info:
        cli.now(conf=fix.CFG_FS_NOT_THERE)  # type: ignore
    assert exec_info.value.code == 1
    out, err = capsys.readouterr()
    assert 'config' in err.lower()
    assert 'is no file' in err.lower()
    assert not out


def test_existing_configuration_file_with_non_json_extension(capsys):
    with pytest.raises(SystemExit) as exec_info:
        cli.now(conf=fix.CFG_FS_NO_JSON_EXTENSION)  # type: ignore
    assert exec_info.value.code == 1
    out, err = capsys.readouterr()
    assert 'config' in err.lower()
    assert 'config has not .json extension' in err.lower()
    assert not out


def test_callback_with_version_false():
    assert cli.callback(False) is None
